import { Injectable } from '@angular/core';
import { MessageService } from './message.service';
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { UserCreds } from './userCreds'
import jwt_decode  from 'jwt-decode'
import { Observable } from 'rxjs';
import { ComplexOuterSubscriber } from 'rxjs/internal/innerSubscribe';



interface TokenTuple {
  authToken: string
  refreshToken: string
}

@Injectable({
  providedIn: 'root'
})
export class TokenService {

  public static Id_AuthToken_Key : string = "id_authtoken";
  public static Id_RefreshToken_Key : string = "id_refreshtoken";

  constructor(private http: HttpClient, private messageService: MessageService) {
  }

  checkAuthenticated(): boolean {
    let result: boolean = false

    const token = localStorage.getItem(TokenService.Id_AuthToken_Key)
    if (token) {
      let expiration = jwt_decode(token)["exp"]
      if ((expiration * 1000) > Date.now()) {
        result = true
      } else {
        this.logout()
      }
    }

    return result
  }

  logout() {
    localStorage.removeItem(TokenService.Id_AuthToken_Key)
    localStorage.removeItem(TokenService.Id_RefreshToken_Key)
    this.messageService.add("Token removed from local storage")
  }

  async login(login: string, password: string) {
    this.messageService.add(`TokenService: trying to login and obtain token`)
    const userCreds : UserCreds = {
      "application": "hv2",
      "login": login,
      "password": password
    }
    const tokenTuple: TokenTuple = await this.http.post<TokenTuple>(
      "https://authservice.hottis.de/refreshable", 
      userCreds
    ).toPromise()
    localStorage.setItem(TokenService.Id_AuthToken_Key, tokenTuple.authToken)
    localStorage.setItem(TokenService.Id_RefreshToken_Key, tokenTuple.refreshToken)
    this.messageService.add("Token saved")
  }

  async refresh() {
    try {
      this.messageService.add(`TokenService: trying to refresh tokens`)
      const refreshToken = localStorage.getItem(TokenService.Id_RefreshToken_Key)
      const tokenTuple: TokenTuple = await this.http.post<TokenTuple>(
        "https://authservice.hottis.de/refresh", 
        refreshToken
      ).toPromise()
      localStorage.setItem(TokenService.Id_AuthToken_Key, tokenTuple.authToken)
      localStorage.setItem(TokenService.Id_RefreshToken_Key, tokenTuple.refreshToken)
      this.messageService.add("Token saved")
    } catch (err) {
      this.messageService.add(`error when trying to refresh: ${ JSON.stringify(err, undefined, 4)}`)
      localStorage.removeItem(TokenService.Id_AuthToken_Key)
      localStorage.removeItem(TokenService.Id_RefreshToken_Key)
      this.messageService.add("Token removed from local storage")
    }
  }

  expiryUpdate() : Observable<number> {
    const exUp = new Observable<number>((observer) => {
      let i = 600

      while (i != 0) {
        observer.next(i)
        i -= 1
        await new Promise(f => setTimeout(f, 1000))
      }
      observer.complete()

      return {
        unsubscribe() {
        }
      }
    })
    return exUp
  }

}
